"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("@feathersjs/errors");
const debug_1 = __importDefault(require("debug"));
const debug = (0, debug_1.default)('authLocalMgnt:checkUnique');
/**
 * This module is usually called from the UI to check username, email, etc. are unique.
 */
function checkUnique(options, identifyUser, ownId, meta, params) {
    return __awaiter(this, void 0, void 0, function* () {
        debug('checkUnique', identifyUser, ownId, meta);
        if (params && "query" in params) {
            params = Object.assign({}, params);
            delete params.query;
        }
        const { app, service } = options;
        const usersService = app.service(service);
        const usersServiceId = usersService.id;
        const errProps = [];
        const keys = Object.keys(identifyUser).filter(key => identifyUser[key] != null);
        try {
            for (let i = 0, ilen = keys.length; i < ilen; i++) {
                const prop = keys[i];
                const _params = Object.assign(Object.assign({}, params), { query: { [prop]: identifyUser[prop].trim(), $limit: 0 }, paginate: { default: 1 } });
                if (ownId != null) {
                    _params.query[usersServiceId] = { $ne: ownId };
                }
                const users = yield usersService.find(_params);
                const length = Array.isArray(users) ? users.length : users.total;
                const isNotUnique = length > 0;
                if (isNotUnique) {
                    errProps.push(prop);
                }
            }
        }
        catch (err) {
            throw new errors_1.BadRequest((meta === null || meta === void 0 ? void 0 : meta.noErrMsg) ? null : 'checkUnique unexpected error.', { errors: { msg: err.message, $className: 'unexpected' } });
        }
        if (errProps.length) {
            const errs = {};
            errProps.forEach(prop => { errs[prop] = 'Already taken.'; });
            throw new errors_1.BadRequest((meta === null || meta === void 0 ? void 0 : meta.noErrMsg) ? null : 'Values already taken.', { errors: errs });
        }
        return null;
    });
}
exports.default = checkUnique;
