"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("@feathersjs/errors");
const debug_1 = __importDefault(require("debug"));
const helpers_1 = require("../helpers");
const debug = (0, debug_1.default)('authLocalMgnt:identityChange');
function identityChange(options, identifyUser, password, changesIdentifyUser, notifierOptions = {}, params) {
    return __awaiter(this, void 0, void 0, function* () {
        // note this call does not update the authenticated user info in hooks.params.user.
        debug('identityChange', password, changesIdentifyUser);
        if (params && "query" in params) {
            params = Object.assign({}, params);
            delete params.query;
        }
        const usersService = options.app.service(options.service);
        const usersServiceId = usersService.id;
        const { delay, identifyUserProps, longTokenLen, shortTokenLen, shortTokenDigits, passwordField, notifier, sanitizeUserForClient } = options;
        (0, helpers_1.ensureObjPropsValid)(identifyUser, identifyUserProps);
        (0, helpers_1.ensureObjPropsValid)(changesIdentifyUser, identifyUserProps);
        const users = (yield usersService.find(Object.assign(Object.assign({}, params), { query: Object.assign(Object.assign({}, identifyUser), { $limit: 2 }), paginate: false })));
        const user = (0, helpers_1.getUserData)(users);
        try {
            yield (0, helpers_1.comparePasswords)(password, user[passwordField]);
        }
        catch (err) {
            throw new errors_1.BadRequest('Password is incorrect.', { errors: { [passwordField]: 'Password is incorrect.', $className: 'badParams' } });
        }
        const [verifyToken, verifyShortToken] = yield Promise.all([
            (0, helpers_1.getLongToken)(longTokenLen),
            (0, helpers_1.getShortToken)(shortTokenLen, shortTokenDigits)
        ]);
        const patchedUser = yield usersService.patch(user[usersServiceId], {
            verifyExpires: Date.now() + delay,
            verifyToken,
            verifyShortToken,
            verifyChanges: changesIdentifyUser
        }, Object.assign({}, params));
        const userResult = yield (0, helpers_1.notify)(notifier, 'identityChange', patchedUser, notifierOptions);
        return sanitizeUserForClient(userResult);
    });
}
exports.default = identityChange;
